﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Collections.Specialized
Imports System.Runtime.CompilerServices
Imports System.Diagnostics

Namespace System.Collections.Generic

   Public Class ResetList(Of T) : Inherits ObservableCollection(Of T)


      Protected Function ChangePropIfDifferent(Of TProp)(value As TProp, ByRef backingField As TProp, <CallerMemberName> Optional name As String = "") As Boolean
         Return ChangePropIfDifferent(value, backingField, {New PropertyChangedEventArgs(name)})
      End Function
      Protected Function ChangePropIfDifferent(Of TProp)(value As TProp, ByRef backingField As TProp, ParamArray propNames() As String) As Boolean
         Return ChangePropIfDifferent(value, backingField, propNames.ToArray(Function(s) New PropertyChangedEventArgs(s)))
      End Function
      Protected Function ChangePropIfDifferent(Of TProp)(value As TProp, ByRef backingField As TProp, ParamArray props() As PropertyChangedEventArgs) As Boolean
         If Object.Equals(backingField, value) Then Return False
         backingField = value
         RaisePropChanged(props)
         Return True
      End Function
      Public Sub RaisePropChanged(ParamArray names() As String)
         RaisePropChanged(names.ToArray(Function(name) New PropertyChangedEventArgs(name)))
      End Sub
      Public Sub RaisePropChanged(ParamArray eventArgses() As PropertyChangedEventArgs)
         RaisePropChanged(DirectCast(eventArgses, IEnumerable(Of PropertyChangedEventArgs)))
      End Sub
      Public Sub RaisePropChanged(eventArgses As IEnumerable(Of PropertyChangedEventArgs))
         VerifyPropertyNames(eventArgses)
         For Each e In eventArgses
            OnPropertyChanged(e)
         Next
      End Sub
      Private Shared _AllProperties As New Dictionary(Of Type, HashSet(Of String))
      <Conditional("DEBUG"), DebuggerStepThrough()> _
      Public Sub VerifyPropertyNames(eventArgses As IEnumerable(Of PropertyChangedEventArgs))
         Dim propNames As HashSet(Of String) = Nothing
         Dim tp = Me.GetType
         If Not _AllProperties.TryGetValue(tp, propNames) Then
            propNames = New HashSet(Of String)(From pd In Me.GetType.GetProperties Select pd.Name)
            _AllProperties.Add(tp, propNames)
         End If
         For Each pe In eventArgses
            If Not propNames.Contains(pe.PropertyName) Then Throw Me.Exception("trying to raise an unknown PropertyChanged")
         Next
      End Sub

      ''' <summary>löscht und befüllt ab startIndex neu. Unter Umgehung der InsertItem-Logik der einzeln zugefügten Items </summary>
      Public Overridable Sub AddRange(news As IEnumerable(Of T), Optional startIndex As Integer = Integer.MaxValue)
         Dim l = InnerList
         Dim cnt = l.Count
         Select Case startIndex.ClipIn(0, cnt)
            Case 0 : l.Clear()
            Case cnt
            Case Else : l.RemoveRange(startIndex, cnt - startIndex)
         End Select
         l.AddRange(news)
         If cnt <> l.Count Then Me.OnPropertyChanged(New PropertyChangedEventArgs("Count"))
         Me.OnPropertyChanged(New PropertyChangedEventArgs("Item[]"))
         RaiseReset()
      End Sub

      Public Sub RaiseReset()
         MyBase.OnCollectionChanged(New NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset))
      End Sub
      Public ReadOnly Property InnerList As List(Of T)
         <DebuggerStepThrough()> _
         Get
            Return DirectCast(Items, List(Of T))
         End Get
      End Property
   End Class

End Namespace